/*
*****************************************************************************************************
-- EXECUTION PLAN FOR SQL STATEMENTS
*****************************************************************************************************

-- REMEBER HOW SQL STATEMENTS ARE PROCESSED BY ORACLE: PARSING, EXECUTION PLAN, FETCH, RESULTS

-> SEE Figure1_SQLProcessing.gif

-- MOST OF THE TIME YOU DON'T NEED TO WORRY TOO MUCH ABOUT THE EXECUTION PLAN (SEE INDEX!)

-- THERE ARE MANY WAYS TO ANALYSING THE EXECUTION PLAN

   -> FIRST, MAKE SURE YOU HAVE THE PLAN_TABLE (CREATE BY @$ORACLE_HOME/rdbms/admin/utlxplan.sql)
   
   SELECT * FROM PLAN_TABLE

   -> NOW RUN THE EXPLAIN PLAN FOR <query>
   
   explain plan for select * from hr.employees
   
   -> CHECK THE EXECUTION PLAN DETAILS. NOTE THE PLAIN_ID COLUMN
   
   SELECT * FROM PLAN_TABLE

  -- MAIN COLUMNS:
          -> PLAN_ID:           UNIQUE IDENTIFIER OF THE PLAN
          -> OPERATION:         THE INTERNAL PART TO EXECUTE THE SQL STATEMENT
          -> OBJECT_NAME:       NAME OF OBJECT USED IN THE OPERATION
          -> CARDINALITY:       NUMBER OF ROWS FOR THE OPERATION
          -> CPU COST:          INDICATE PROCESSING
          -> IO COST:           INDICATE IO OPERATIONS
          -> PROJECTION:        THE COLUMNS RETURNED
          -> FILTER_PREDICATES: THE FILTER OPTIONS

  -- YOU CAN ALSO SEE THE LAST EXECUTION PLAN WITH THE HELP OF THE dbms_xplan PACKAGE:
  
  select * from hr.employees
  
  -- SEE THE PLAN FOR THE PREVIOUS SQL
  
  SELECT PLAN_TABLE_OUTPUT 
  FROM TABLE(DBMS_XPLAN.DISPLAY());
  
  -> LET'S SEE A MORE COMPLEX EXECUTION PLAN
  
  EXPLAIN PLAN FOR  
  SELECT emp1.first_name ||' ' ||emp1.last_name AS employee_name,
                              emp1.salary      AS employee_salary,
       emp2.first_name ||' ' ||emp2.last_name AS manager_name,
       emp2.salary                            AS manager_salary
  FROM       HR.employees emp1
  INNER JOIN HR.employees emp2
  ON   emp1.manager_id = emp2.employee_id
    AND emp1.salary    > emp2.salary;



  SELECT PLAN_TABLE_OUTPUT 
  FROM TABLE(DBMS_XPLAN.DISPLAY());
  
  -- THERE CAN BE MANY OPTIMIZER OPERATIONS INSIDE A EXECUTION PLAN
  -- THE DBA SHOULD KNOW THE MAIN ONES. FOR A GOOD DESCRIPTION CHECK:
  
  http://docs.oracle.com/cd/B19306_01/server.102/b14211/optimops.htm#i73898
  
  -- THE PREDICATES ARE AN IMPORTANT PART OF THE PLAN
  -- THEY CORRESPONT TO FILTERS AND JOIN CONDITIONS
    
  explain plan for
  select * from hr.employees where employee_id > 105
  
  SELECT * FROM PLAN_TABLE
  
  -- NOTE: WE DON'T CHANGE THE PLAN DIRECTELY 
  --       WE CHANGE THE SQL THAT WILL GENERATE A NEW EXECUTION PLAN